using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;

using EQIA.EQIDL;
using System.Windows.Forms;

namespace EQIA
{
    public class EQInterface : IEQInterface
    {
        #region Static Data

        private static readonly string DefaultEquiName = @"default";
        private static readonly string FilenameEqui = @"\EQUI.xml";
        private static readonly string XmlCompositeNodeName = @"Composite";
        private static readonly string XmlIncludeNodeName = @"Include";
        private static readonly string XmlSidlNodeText = @"sidl.xml";
        private static readonly string XmlDirUpFoward = @"../";
        private static readonly string XmlDirUpBack = @"..\";

        /// <summary>
        /// A list of included xml files used in the default interface.
        /// </summary>
        public static List<string> DefaultIncludes = new List<string>();

        #endregion

        #region Fields

        private IEQInterface myParent;

        private DirectoryInfo myDirectory;
        private Dictionary<string, EQInterfaceFile> myFiles;
        private Dictionary<string, Class> myElements;

        /// <summary>
        /// Name of the interface this one will fall back on for missing elements.
        /// </summary>
        public string ReferenceInterfaceName;

        #endregion

        #region Constructor

        /// <summary>
        /// Create a new interface.
        /// </summary>
        /// <param name="directory">Full path to the directory that houses this interface.</param>
        public EQInterface(string directory)
        {
            myDirectory = new DirectoryInfo(directory);
            myFiles = new Dictionary<string, EQInterfaceFile>();
            myElements = new Dictionary<string, Class>();
        }

        #endregion

        #region Listing Methods

        public void AddFile(EQInterfaceFile newFile)
        {
            newFile.Interface = this;
            myFiles[newFile.Filename] = newFile;
        }

        public void RemoveFile(string filename)
        {
            if (myFiles.ContainsKey(filename))
                myFiles.Remove(filename);
        }

        public void AddElement(Class newElement)
        {
            newElement.Interface = this;
            myElements[newElement.Name] = newElement;
        }

        public void RemoveElement(string elementName)
        {
            if (myElements.ContainsKey(elementName))
                myElements.Remove(elementName);
        }

        public Class GetElement(string elementName)
        {
            return myElements.ContainsKey(elementName) ? myElements[elementName] : null;
        }

        public T GetElement<T>(string elementName)
        {
            if (myElements.ContainsKey(elementName))
            {
                try { return (T)((object)myElements[elementName]); }
                catch (InvalidCastException) { return default(T); }
            }
            return default(T);
        }

        #endregion

        #region Xml Methods

        public void Deserialize()
        {
            if (myDirectory == null) return;

            bool isDefault = (myDirectory.Name == DefaultEquiName);
            FileInfo equi = new FileInfo(myDirectory.FullName + FilenameEqui);

            if (equi.Exists)
            {
                XmlDocument xml = new XmlDocument(); xml.Load(equi.FullName);
                XmlNode root = xml.DocumentElement;

                for (int i = 0; i < root.ChildNodes.Count; i++)
                    if (root.ChildNodes[i].LocalName == XmlCompositeNodeName)
                        for (int j = 0; j < root.ChildNodes[i].ChildNodes.Count; j++)
                            if (root.ChildNodes[i].ChildNodes[j].LocalName == XmlIncludeNodeName &&
                                root.ChildNodes[i].ChildNodes[j].InnerText.ToLower() != XmlSidlNodeText)
                            {
                                string nodeText = root.ChildNodes[i].ChildNodes[j].InnerText;
                                if (nodeText.Contains(XmlDirUpBack) || nodeText.Contains(XmlDirUpFoward))
                                {
                                    FetchReferenceInterface(ParseIncludeString(nodeText));
                                    continue;
                                }
                                if (isDefault)
                                    DefaultIncludes.Add(nodeText);

                                EQInterfaceFile newFile = new EQInterfaceFile(this, myDirectory.FullName + "\\" + nodeText);
                                newFile.Deserialize();
                                AddFile(newFile);
                            }
            }
            else
            {
                for (int i = 0; i < DefaultIncludes.Count; i++)
                {
                    FileInfo file = new FileInfo(myDirectory.FullName + "\\" + DefaultIncludes[i]);
                    if (file.Exists)
                    {
                        EQInterfaceFile newFile = new EQInterfaceFile(this, file.FullName);
                        newFile.Deserialize();
                        AddFile(newFile);
                    }
                }
            }
        }

        private void FetchReferenceInterface(string name)
        {
        }

        private static string ParseIncludeString(string xmlValue)
        {
            if (xmlValue.Contains(@"\")) xmlValue.Replace('\\', '/');
            return xmlValue.Split('/')[1];
        }

        #endregion

        #region Conversion Methods

        public override string ToString()
        {
            return Name;
        }

        #endregion

        #region IEqInterface Members

        /// <summary>
        /// The interface this one will fall back on for missing elements.
        /// </summary>
        public IEQInterface Parent { get { return myParent; } }

        /// <summary>
        /// A list of elements in the interface.
        /// </summary>
        public Dictionary<string, Class> Elements { get { return myElements; } }

        /// <summary>
        /// A list of files in the interface.
        /// </summary>
        public Dictionary<string, EQInterfaceFile> Files { get { return myFiles; } }

        /// <summary>
        /// The name of the interface's folder.
        /// </summary>
        public string Name { get { return myDirectory != null ? myDirectory.Name : string.Empty; } }

        /// <summary>
        /// The full path to the interface's folder.
        /// </summary>
        public string Path { get { return myDirectory != null ? myDirectory.FullName : string.Empty; } }

        #endregion
    }
}
